package dev.fitko.fitconnect.api.config;

import dev.fitko.fitconnect.api.config.defaults.SchemaConfig;
import dev.fitko.fitconnect.api.exceptions.client.FitConnectInitialisationException;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.With;
import lombok.experimental.Accessors;

import java.net.InetSocketAddress;
import java.net.Proxy;
import java.net.URI;
import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

@Getter
@Builder
@Accessors(chain = true)
@NoArgsConstructor
@AllArgsConstructor
public class ApplicationConfig {

    public static final String AUTH_TAG_SPLIT_TOKEN = "\\.";

    private ProxyConfig proxyConfig;

    @Builder.Default
    private URI setSchemaWriteVersion = SchemaConfig.SET_V_1_2_1.getSchemaUri();

    @Builder.Default
    private URI metadataSchemaWriteVersion = SchemaConfig.METADATA_V_1_2_0.getSchemaUri();

    @Builder.Default
    private URI destinationSchemaVersion = SchemaConfig.XZUFI_DESTINATION_SCHEMA.getSchemaUri();

    @Builder.Default
    private Map<String, String> submissionDataSchemas = new HashMap<>();

    @Deprecated(since = "2.0.0")
    private String submissionDataSchemaPath;

    private SenderConfig senderConfig;

    private SubscriberConfig subscriberConfig;

    @With
    @Builder.Default
    private Map<EnvironmentName, Environment> environments = new HashMap<>();

    private EnvironmentName activeEnvironment;

    public boolean isAutoRejectEnabled() {
        return getCurrentEnvironment().getEnableAutoReject();
    }

    public boolean isSkipSubmissionDataValidation() {
        return getCurrentEnvironment().getSkipSubmissionDataValidation();
    }

    public boolean isAllowInsecurePublicKey() {
        return getCurrentEnvironment().getAllowInsecurePublicKey();
    }

    public Environment getCurrentEnvironment() {
        if (environments.containsKey(activeEnvironment)) {
            return environments.get(activeEnvironment);
        } else {
            throw new FitConnectInitialisationException("No environment with name '" + activeEnvironment.getName() + "' found. Available environments are: " + getAvailableEnvironmentNames());
        }
    }

    public String getSubmissionBaseUrl() {
        return getCurrentEnvironment().getSubmissionBaseUrls().stream().findFirst().orElseThrow(() -> new FitConnectInitialisationException("No submission base url found, expected at least one"));
    }

    public String getSelfServicePortalBaseUrl() {
        return getCurrentEnvironment().getSelfServicePortalBaseUrl();
    }

    public String getRoutingBaseUrl() {
        return getCurrentEnvironment().getRoutingBaseUrl();
    }

    public String getAuthBaseUrl() {
        return getCurrentEnvironment().getAuthBaseUrl();
    }

    private String getAvailableEnvironmentNames() {
        return environments.keySet().stream().map(EnvironmentName::getName).collect(Collectors.joining(" | "));
    }

    public boolean isProxySet() {
        return proxyConfig != null && proxyConfig.isProxySet();
    }

    public Proxy getProxy() {
        if (!isProxySet()) {
            return Proxy.NO_PROXY;
        }
        return new Proxy(Proxy.Type.HTTP, new InetSocketAddress(proxyConfig.getHttpProxyHost(), proxyConfig.getHttpProxyPort()));
    }
}

